==> 0.0.8.tar.gz: lib/src/open_weather_map.dart <==
import 'dart:async';
import 'dart:convert';

import 'package:flutter/foundation.dart';
import 'package:open_weather_map_client/src/constants.dart';
import 'package:open_weather_map_client/src/model/city.dart';
import 'package:open_weather_map_client/src/model/country.dart';
import 'package:open_weather_map_client/src/model/weather.dart';
import 'package:http/http.dart' as http;

/// Useful to obtain weather data, it connects with the Open Weather Map API.
class OpenWeatherMap {
  /// The key obtained from the Open Weather Map API.
  final String apiKey;

  /// The language to get the data from the API.
  final Language language;

  /// Controls the maximum time in seconds to wait to get the data.
  /// If you have a slow connection make sure to use the correct seconds.
  ///
  /// Default is 5 seconds
  final int requestTimeout;

  /// Current weather city.
  City? _city;

  OpenWeatherMap({
    required this.apiKey,
    this.language = Language.english,
    this.requestTimeout = 5,
  });

  /// Response request body from network as String
  Future<String> _response(Uri uri) async {
    return http
        .get(uri)
        .timeout(Duration(seconds: requestTimeout),
            onTimeout: () => throw TimeoutException(
                'The connection has timed out, Please try again!'))
        .then(
          (value) => value.body,
        );
  }

  /// Returns the current weather by city information.
  ///
  /// The optional [id] parameter is the city ID used by API to get current
  /// weather; the oprional [countryCode] parameter work with required city
  /// [name] for more precision.
  Future<Weather> currentWeatherByCity({
    int? id,
    String? countryCode,
    required String name,
  }) async {
    final Uri uri = Uri.https(
      apiAuthority,
      '$apiUnencodedPath$apiCurrentWeatherPath',
      _parameters(cityId: id, countryCode: countryCode, cityName: name),
    );
    final Map<City, Weather> map = await compute(
      _parseWeather,
      await _response(uri),
    );
    _city = map.keys.first;
    return Future.value(map.values.first);
  }

  /// Return the current weather by [latitude] and [longitude] required parameters.
  Future<Weather> currentWeatherByLocation({
    required double latitude,
    required double longitude,
  }) async {
    final Uri uri = Uri.https(
      apiAuthority,
      '$apiUnencodedPath$apiCurrentWeatherPath',
      _parameters(latitude: latitude, longitude: longitude),
    );
    final Map<City, Weather> map = await compute(
      _parseWeather,
      await _response(uri),
    );
    _city = map.keys.first;
    return Future.value(map.values.first);
  }

  /// Returns the five day forecast as a List<Weather>.
  ///
  /// The optional [id] parameter is the city ID used by API to get current
  /// weather; the oprional [countryCode] parameter work with required city
  /// [name] for more precision.
  Future<List<Weather>> forecastByCity({
    int? id,
    String? countryCode,
    required String name,
  }) async {
    final Uri uri = Uri.https(
      apiAuthority,
      '$apiUnencodedPath$apiFiveDayForecastPath',
      _parameters(cityId: id, countryCode: countryCode, cityName: name),
    );
    return compute(_parseForecast, await _response(uri));
  }

  /// Returns the five day forecast by [latitude] and [longitude] required parameters.
  Future<List<Weather>> forecastByLocation({
    required double latitude,
    required double longitude,
  }) async {
    final Uri uri = Uri.https(
      apiAuthority,
      '$apiUnencodedPath$apiFiveDayForecastPath',
      _parameters(latitude: latitude, longitude: longitude),
    );
    return compute(_parseForecast, await _response(uri));
  }

  /// Parameters to use in URI for get request to API
  Map<String, String> _parameters({
    int? cityId,
    String? countryCode,
    String? cityName,
    double? latitude,
    double? longitude,
  }) {
    String? city = cityName;
    if (city != null && countryCode != null) city += ',$countryCode';
    return {
      'appid': apiKey,
      if (cityId != null) 'id': cityId.toString(),
      if (city != null && cityId == null) 'q': city,
      if (city == null && cityId == null) 'lat': latitude.toString(),
      if (city == null && cityId == null) 'lon': longitude.toString(),
      'lang': languageCode(language),
    };
  }

  /// Returns the current detailed city.
  ///
  /// If you do not call the [currentWeatherByCity] or [currentWeatherByLocation]
  /// method returns null, the city is loaded when obtaining the current weather.
  City? get detailedCity => _city;
}

/// Used for compute isolate to parse JSON to Weather class.
Map<City, Weather> _parseWeather(String responseBody) {
  final parsed = jsonDecode(responseBody);
  return {
    City(
      id: parsed['id'] as int,
      country: Country.fromCode(parsed['sys']['country'] as String),
      latitude: double.parse(parsed['coord']['lat'].toString()),
      longitude: double.parse(parsed['coord']['lon'].toString()),
      name: parsed['name'] as String?,
      population: parsed['population'] as int?,
      sunrise: DateTime.fromMillisecondsSinceEpoch(
        parsed['sys']['sunrise'] * 1000 as int,
      ),
      sunset: DateTime.fromMillisecondsSinceEpoch(
        parsed['sys']['sunset'] * 1000 as int,
      ),
    ): Weather(
      clouds: parsed['clouds']['all'] as int?,
      condition: parsed['weather'][0]['main'] as String,
      dateTime: DateTime.fromMillisecondsSinceEpoch(parsed['dt'] * 1000 as int),
      description: parsed['weather'][0]['description'] as String,
      groundLevel: parsed['main']['grnd_level'] as int?,
      humidity: parsed['main']['humidity'] as int,
      iconId: parsed['weather'][0]['icon'] as String?,
      maxTemperature: double.parse(parsed['main']['temp_max'].toString()),
      minTemperature: double.parse(parsed['main']['temp_min'].toString()),
      pop:
          parsed['pop'] != null ? double.parse(parsed['pop'].toString()) : null,
      pressure: parsed['main']['pressure'] as int,
      rainLastHour: parsed['rain']?['1h'] != null
          ? double.parse(parsed['rain']['1h'].toString())
          : null,
      rainLastThreeHours: parsed['rain']?['3h'] != null
          ? double.parse(parsed['rain']['3h'].toString())
          : null,
      seaLevel: parsed['main']['sea_level'] as int?,
      temperature: double.parse(parsed['main']['temp'].toString()),
      temperatureFeelsLike: parsed['main']['feels_like'] != null
          ? double.parse(parsed['main']['feels_like'].toString())
          : null,
      visibility: parsed['visibility'] as int?,
      windDirection: parsed['wind']['deg'] as int?,
      windGust: parsed['wind']?['gust'] != null
          ? double.parse(parsed['wind']['gust'].toString())
          : null,
      windSpeed: double.parse(parsed['wind']['speed'].toString()),
    )
  };
}

/// Used for compute isolate to parse JSON to List<Weather>.
List<Weather> _parseForecast(String responseBody) {
  final parsed = jsonDecode(responseBody);
  return (parsed['list'] as List<dynamic>)
      .map<Weather>(
        (json) => Weather(
          clouds: json['clouds']['all'] as int?,
          condition: json['weather'][0]['main'] as String,
          dateTime:
              DateTime.fromMillisecondsSinceEpoch(json['dt'] * 1000 as int),
          description: json['weather'][0]['description'] as String,
          groundLevel: json['main']['grnd_level'] as int?,
          humidity: json['main']['humidity'] as int,
          iconId: json['weather'][0]['icon'] as String?,
          maxTemperature: double.parse(json['main']['temp_max'].toString()),
          minTemperature: double.parse(json['main']['temp_min'].toString()),
          pop:
              json['pop'] != null ? double.parse(json['pop'].toString()) : null,
          pressure: json['main']['pressure'] as int,
          rainLastHour: json['rain']?['1h'] != null
              ? double.parse(json['rain']['1h'].toString())
              : null,
          rainLastThreeHours: json['rain']?['3h'] != null
              ? double.parse(json['rain']['3h'].toString())
              : null,
          seaLevel: json['main']['sea_level'] as int?,
          temperature: double.parse(json['main']['temp'].toString()),
          temperatureFeelsLike: json['main']['feels_like'] != null
              ? double.parse(json['main']['feels_like'].toString())
              : null,
          visibility: json['visibility'] as int?,
          windDirection: json['wind']['deg'] as int?,
          windGust: json['wind']?['gust'] != null
              ? double.parse(json['wind']['gust'].toString())
              : null,
          windSpeed: double.parse(json['wind']['speed'].toString()),
        ),
      )
      .toList();
}

==> 0.1.4.tar.gz: lib/src/open_weather_map.dart <==
import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:open_weather_map_client/src/common/constants.dart';
import 'package:open_weather_map_client/src/common/parse_data.dart';
import 'package:open_weather_map_client/src/model/city.dart';
import 'package:open_weather_map_client/src/model/weather.dart';
import 'package:http/http.dart' as http;

/// Useful to obtain weather data, it connects with the Open Weather Map API.
class OpenWeatherMap {
  /// The key obtained from the Open Weather Map API.
  final String apiKey;

  /// The language to get the data from the API.
  final Language language;

  /// The language code to get description from API.
  final String? langCode;

  /// Controls the maximum time in seconds to wait to get the data.
  /// If you have a slow connection make sure to use the correct seconds.
  ///
  /// Default is 5 seconds
  final int requestTimeout;

  /// Current weather city.
  Future<City>? _city;

  /// Creates a OpenWeatherMap instance.
  ///
  /// The [apiKey] is required to connect with API, The [language] parameter its
  /// default value is English and [requestTimeout] parameter its default value is 5.
  OpenWeatherMap({
    required this.apiKey,
    this.language = Language.english,
    this.langCode,
    this.requestTimeout = 5,
  });

  /// Parameters to use in URI for get request to API
  Map<String, String> _parameters({
    int? cityId,
    String? cityName,
    String? countryCode,
    double? latitude,
    double? longitude,
  }) {
    String? city = cityName;
    if (city != null && countryCode != null) {
      city += ',$countryCode';
    }
    return {
      'appid': apiKey,
      if (cityId != null) 'id': cityId.toString(),
      if (city != null && cityId == null) 'q': city,
      if (city == null && cityId == null) 'lat': latitude.toString(),
      if (city == null && cityId == null) 'lon': longitude.toString(),
      'lang': langCode ?? languageCode(language),
    };
  }

  /// Response request body from network as String
  Future<String> _response(Uri uri) async {
    return http
        .get(uri)
        .timeout(Duration(seconds: requestTimeout),
            onTimeout: () => throw TimeoutException(
                'The connection has timed out, Please try again!'))
        .then(
          (value) => value.body,
        );
  }

  /// Returns the current weather by city information.
  ///
  /// The optional [id] parameter is the city ID used by API to get current
  /// weather; the oprional [countryCode] parameter work with required city
  /// [name] for more precision.
  Future<Weather> currentWeatherByCity({
    int? id,
    String? countryCode,
    required String name,
  }) async {
    final String responseBody = await _response(
      Uri.https(
        apiAuthority,
        '$apiUnencodedPath$apiCurrentWeatherPath',
        _parameters(cityId: id, countryCode: countryCode, cityName: name),
      ),
    );
    _city ??= compute(parseCity, responseBody);
    return compute(parseWeather, responseBody);
  }

  /// Return the current weather by [latitude] and [longitude] required parameters.
  Future<Weather> currentWeatherByLocation({
    required double latitude,
    required double longitude,
  }) async {
    final String responseBody = await _response(
      Uri.https(
        apiAuthority,
        '$apiUnencodedPath$apiCurrentWeatherPath',
        _parameters(latitude: latitude, longitude: longitude),
      ),
    );
    _city ??= compute(parseCity, responseBody);
    return compute(parseWeather, responseBody);
  }

  /// Returns the five day forecast as a List<Weather>.
  ///
  /// The optional [id] parameter is the city ID used by API to get current
  /// weather; the oprional [countryCode] parameter work with required city
  /// [name] for more precision.
  Future<List<Weather>> forecastByCity({
    int? id,
    String? countryCode,
    required String name,
  }) async {
    final Uri uri = Uri.https(
      apiAuthority,
      '$apiUnencodedPath$apiFiveDayForecastPath',
      _parameters(cityId: id, countryCode: countryCode, cityName: name),
    );
    return compute(parseForecast, await _response(uri));
  }

  /// Returns the five day forecast by [latitude] and [longitude] required parameters.
  Future<List<Weather>> forecastByLocation({
    required double latitude,
    required double longitude,
  }) async {
    final Uri uri = Uri.https(
      apiAuthority,
      '$apiUnencodedPath$apiFiveDayForecastPath',
      _parameters(latitude: latitude, longitude: longitude),
    );
    return compute(parseForecast, await _response(uri));
  }

  /// Returns the current detailed city.
  ///
  /// If you do not call the [currentWeatherByCity] or [currentWeatherByLocation]
  /// method returns null, the city is loaded when obtaining the current weather.
  Future<City>? get detailedCity => _city;
}
